@@ -10,6 +10,11 @@ from draftsman.error import DraftsmanError
 from schema import SchemaError
 from typing import Union
 
+from typing import TYPE_CHECKING
+
+if TYPE_CHECKING:  # pragma: no coverage
+    from draftsman.classes.entity import Entity
+
 
 class EightWayDirectionalMixin(object):
     """
@@ -28,15 +33,27 @@ class EightWayDirectionalMixin(object):
         self._rotatable = True
 
         # Keep track of the entities width and height regardless of rotation
-        self.static_tile_width = self.tile_width
-        self.static_tile_height = self.tile_height
-        self.static_collision_box = self.collision_box
+        self._static_tile_width = self.tile_width
+        self._static_tile_height = self.tile_height
+        self._static_collision_set = self.collision_set
+
+        if not hasattr(self, "_overwritten_collision_set"):
+            self._collision_set_rotation = {}
+            if hasattr(self, "_disable_collision_set_rotation"):  # pragma: no branch
+                # Set every collision orientation to the single collision_set
+                for i in range(8):
+                    self._collision_set_rotation[i] = self.collision_set
+            # else:
+            #     # Automatically generate a set of rotated collision sets for every
+            #     # orientation
+            #     for i in range(8):
+            #         self._collision_set_rotation[i] = self._collision_set.rotate(i)
 
         self.direction = 0
         if "direction" in kwargs:
             self.direction = kwargs["direction"]
             self.unused_args.pop("direction")
-        self._add_export("direction", lambda x: x != 0)
+        self._add_export("direction", lambda x: x != 0, lambda k, v: (k, int(v)))
 
         # Technically redundant, but we reset the position if the direction has
         # changed to reflect its changes
@@ -84,22 +101,37 @@ class EightWayDirectionalMixin(object):
         else:
             self._direction = Direction(value)
 
+        # if self._direction in {2, 3, 6, 7}:
+        #     self._tile_width = self.static_tile_height
+        #     self._tile_height = self.static_tile_width
+        #     self._collision_box[0] = [
+        #         self.static_collision_box[0][1],
+        #         self.static_collision_box[0][0],
+        #     ]
+        #     self._collision_box[1] = [
+        #         self.static_collision_box[1][1],
+        #         self.static_collision_box[1][0],
+        #     ]
+        # else:
+        #     self._tile_width = self.static_tile_width
+        #     self._tile_height = self.static_tile_height
+        #     self._collision_box = self.static_collision_box
+
+        self._collision_set = self._collision_set_rotation[self._direction]
+        # TODO: do this a little more reliably
         if self._direction in {2, 3, 6, 7}:
-            self._tile_width = self.static_tile_height
-            self._tile_height = self.static_tile_width
-            self._collision_box[0] = [
-                self.static_collision_box[0][1],
-                self.static_collision_box[0][0],
-            ]
-            self._collision_box[1] = [
-                self.static_collision_box[1][1],
-                self.static_collision_box[1][0],
-            ]
+            self._tile_width = self._static_tile_height
+            self._tile_height = self._static_tile_width
         else:
-            self._tile_width = self.static_tile_width
-            self._tile_height = self.static_tile_height
-            self._collision_box = self.static_collision_box
+            self._tile_width = self._static_tile_width
+            self._tile_height = self._static_tile_height
 
         # Reset the grid/absolute positions in case the direction changed
-        # self.set_tile_position(self.tile_position[0], self.tile_position[1])
-        self.tile_position = (self.tile_position["x"], self.tile_position["y"])
+        self.tile_position = (self.tile_position.x, self.tile_position.y)
+
+    # =========================================================================
+
+    def mergable_with(self, other):
+        # type: (Entity) -> bool
+        base_mergable = super(EightWayDirectionalMixin, self).mergable_with(other)
+        return base_mergable and self.direction == other.direction
