@@ -4,7 +4,11 @@
 from __future__ import unicode_literals
 
 from draftsman.classes.entity import Entity
-from draftsman.classes.mixins import CircuitConnectableMixin, InventoryMixin
+from draftsman.classes.mixins import (
+    RequestItemsMixin,
+    CircuitConnectableMixin,
+    InventoryMixin,
+)
 from draftsman.warning import DraftsmanWarning
 
 from draftsman.data.entities import containers, raw
@@ -12,7 +16,7 @@ from draftsman.data.entities import containers, raw
 import warnings
 
 
-class Container(CircuitConnectableMixin, InventoryMixin, Entity):
+class Container(InventoryMixin, RequestItemsMixin, CircuitConnectableMixin, Entity):
     """
     An entity that holds items.
     """
@@ -21,8 +25,6 @@ class Container(CircuitConnectableMixin, InventoryMixin, Entity):
         # type: (str, **dict) -> None
         super(Container, self).__init__(name, containers, **kwargs)
 
-        self._inventory_bar_enabled = raw[self.name].get("enable_inventory_bar", True)
-
         for unused_arg in self.unused_args:
             warnings.warn(
                 "{} has no attribute '{}'".format(type(self), unused_arg),
